Attribute VB_Name = "basMain"
Option Explicit

' ***************************************************************************
' Module:        basMain
'
' Description:   These are the common routines you will need to run most
'                applications
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
' ***************************************************************************
  
' ---------------------------------------------------------------------------
' Global variables
' ---------------------------------------------------------------------------
  Public g_strVersion        As String
  Private m_strTargetTitle   As String
  Private m_blnFoundApp      As Boolean
  Private m_intAppCount      As Integer
  
' ---------------------------------------------------------------------------
' Constants ("ThunderMain"    - VBx Development)
'           ("ThunderRT6Main" - VB6 Executable File)
' ---------------------------------------------------------------------------
  Private Const PGM_CLASS    As String = "ThunderRT6Main"
  Private Const PGM_TITLE    As String = "Reboot"        ' Exe name w/o ext
  Private Const PGM_NAME     As String = "Reboot"        ' Caption title
  Private Const MAX_PATH     As Long = 260

' ---------------------------------------------------------------------------
' Declares required for finding a previous instance of an active program
' ---------------------------------------------------------------------------
  Private Declare Function EnumWindows Lib "user32" _
          (ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
  
  Private Declare Function GetWindowText Lib "user32" _
          Alias "GetWindowTextA" (ByVal hwnd As Long, _
          ByVal lpString As String, ByVal cch As Long) As Long

  Private Declare Function GetClassName Lib "user32" _
          Alias "GetClassNameA" (ByVal hwnd As Long, _
          ByVal lpClassName As String, ByVal nMaxCount As Long) As Long

Public Function FindApplication(ByVal hwnd As Long, _
                                Optional ByVal lParam As Long = 0) As Long

' ***************************************************************************
' Routine:       FindApplication
'
' Description:   This routine will search ALL active programs running under
'                Windows, including the hidden and minimized.  It will
'                look for the parent name.  The partial/full title name will
'                will be used for the search pattern.
'
' Parameters:    hwnd - Generic application handle to check all active
'                       programs
'                lParam - Not used (but required for callbacks)
'
' Returns:       Sets an external flag to TRUE/FALSE based on the findings.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngLength      As Long    ' Length of the title string
  Dim strClassName   As String  ' application Class name string after formatting
  Dim strAppTitle    As String  ' application Parent title string after formatting

' ---------------------------------------------------------------------------
' Initialize return areas with spaces
' ---------------------------------------------------------------------------
  strClassName = Space$(MAX_PATH)
  strAppTitle = Space$(MAX_PATH)
   
' ---------------------------------------------------------------------------
' Make API calls to get the title of the parent application
' ---------------------------------------------------------------------------
  Call GetClassName(hwnd, strClassName, MAX_PATH)
  strClassName = Trim$(Left$(strClassName, Len(PGM_CLASS)))
  
  lngLength = GetWindowText(hwnd, strAppTitle, MAX_PATH)
  strAppTitle = StrConv(Left$(strAppTitle, lngLength), vbLowerCase)
  strAppTitle = LTrim$(strAppTitle)  ' remove all leading blanks
  
' ---------------------------------------------------------------------------
' See if the class name matches.  If it does then check the parent title.
' ---------------------------------------------------------------------------
  If StrComp(strClassName, PGM_CLASS, vbTextCompare) = 0 Then
             
      ' See if this is the right title. Since we may only have a
      ' partial title, then we have to do an Instr() compare.
      ' This is why we want to make sure that the search title is
      ' as unique as possible.
      If InStr(1, strAppTitle, m_strTargetTitle, vbTextCompare) > 0 Then
                
          ' increment the counter.
          m_intAppCount = m_intAppCount + 1
          
          ' If we find more than one occurance of this program
          ' then set the flag to TRUE and leave
          If m_intAppCount > 1 Then
              ' set the flag denoting that we have found a duplicate
              m_blnFoundApp = True
              Exit Function
          End If
      End If
  End If
  
' ---------------------------------------------------------------------------
' Continue searching.
' ---------------------------------------------------------------------------
  FindApplication = 1

End Function

Private Function AlreadyRunning(strAppTitle As String) As Boolean

' ***************************************************************************
' Routine:       AlreadyRunning
'
' Description:   This routine will set an external search flag to FALSE and
'                perform an enumeration of all active programs.
'
' Parameters:    strAppTitle - partial/full name of application title to
'                              look for
'
' Returns:       TRUE/FALSE based on the findings.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngRetCode As Long

' ---------------------------------------------------------------------------
' Intialize variables
' ---------------------------------------------------------------------------
  m_blnFoundApp = False
  m_intAppCount = 0

' ---------------------------------------------------------------------------
' Search all active applicatios to see if this one is already running
' ---------------------------------------------------------------------------
  m_strTargetTitle = StrConv(strAppTitle, vbLowerCase)
  Call EnumWindows(AddressOf FindApplication, &H0)

' ---------------------------------------------------------------------------
' Return TRUE/FALSE based on findings
' ---------------------------------------------------------------------------
  AlreadyRunning = m_blnFoundApp
  
End Function

Sub Main()

' ***************************************************************************
' Routine:       Main
'
' Description:   This is a generic routine to start an application
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  ChDrive App.Path
  ChDir App.Path
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  g_strVersion = PGM_NAME & " v" & App.Major & "." & App.Minor
  
' ---------------------------------------------------------------------------
' See if there is another instance of this program running.  The parameter
' being passed is the name of the EXE without the extension.
' ---------------------------------------------------------------------------
  If AlreadyRunning(PGM_TITLE) Then
      End
  End If
  
' ---------------------------------------------------------------------------
' Load the main form
' ---------------------------------------------------------------------------
  Load frmReboot
  
End Sub

Public Sub StopTheProgram()

' ***************************************************************************
' Routine:       StopTheProgram
'
' Description:   This routine will performt he shutdown process for this
'                application.  If there are any global object/class (not
'                forms) they will be listed below and set to NOTHING so as
'                to free them from memory.  The last task is to unload
'                all form objects.  Then terminate this application.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Set all global objects to nothing
' EXAMPLE:    Set g_obgMyObj = Nothing
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Upload all forms from memory and terminate this application
' ---------------------------------------------------------------------------
  Unload_All_Forms
  End
  
End Sub

Public Sub Unload_All_Forms()

' ***************************************************************************
' Routine:       StopTheProgram
'
' Description:   Unload all active forms associated with this application.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim frm As Form
  
' ---------------------------------------------------------------------------
' Loop thru all the active forms associated with this application
' ---------------------------------------------------------------------------
  For Each frm In Forms
      frm.Hide            ' hide the form
      Unload frm          ' deactivate the form
      Set frm = Nothing   ' free form object from memory
                          ' (prevents memory fragmenting)
  Next
  
End Sub

Public Sub DoShutdownProcess(intAction As Integer)
' This routine is not the normal part of my Main.bas module

  Dim cReboot As clsReboot             ' define class object
  Set cReboot = New clsReboot          ' instantsiate reboot class
  cReboot.ShutdownSystem intAction     ' Start shutdown process
  Set cReboot = Nothing                ' free reboot class from memory
  End                                  ' terminate this application
  
End Sub
