VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsReboot"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' ***************************************************************************
' Module:        clsReboot
'
' Description:   This class is used to shutdown, logoff and/or reboot a
'                system.  Shutting down a Windows 9x, NT, ME, or 2000
'                system is very straightforward.
'
' Thanks to:     Arjang  qwadrox@gmx.net
'                Got some good coding ideas from his examples on the web.
'
' Special Note:  On some Windows 9x operating systems, if the action
'                requested is LOGOFF then the PC will power off.  This does
'                not happen to all machines, just some of them.  Any answers
'                out there?  Please write.
'
'                Example:  CompUSA 500 mhz and Dell 200 mhz will power off
'                          Dell 500 mhz will Logoff.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 02-JUL-1998  Kenneth Ives  kenaso@home.com
' ***************************************************************************
Option Explicit

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Private m_blnWindowsNT      As Boolean
  Private m_blnWindows2000    As Boolean
  Private m_blnErrorDetected  As Boolean
  
' ---------------------------------------------------------------------------
' To Shutdown Windows:
' ---------------------------------------------------------------------------
  Private Const EWX_LOGOFF              As Long = 0
  Private Const EWX_SHUTDOWN            As Long = 1
  Private Const EWX_REBOOT              As Long = 2
  Private Const EWX_FORCE               As Long = 4
  Private Const EWX_POWEROFF            As Long = 8
  Private Const EWX_FORCEIFHUNG         As Long = 16
  Private Const EWX_RESET               As Long = EWX_REBOOT Or _
                                                  EWX_FORCE
  Private Const EWX_POWERDOWN           As Long = EWX_POWEROFF Or _
                                                  EWX_SHUTDOWN
  Private Const ENDSESSION_LOGOFF       As Long = &H80000000
  
' ---------------------------------------------------------------------------
' To Determine the operating system
' ---------------------------------------------------------------------------
  Private Const MAX_SIZE                    As Long = 128
  Private Const VER_PLATFORM_WIN32s         As Long = 0
  Private Const VER_PLATFORM_WIN32_WINDOWS  As Long = 1
  Private Const VER_PLATFORM_WIN32_NT       As Long = 2

  Private Type OSVERSIONINFO
          dwOSVersionInfoSize  As Long
          dwMajorVersion       As Long
          dwMinorVersion       As Long
          dwBuildNumber        As Long
          dwPlatformId         As Long
          szCSDVersion         As String * 128
  End Type

' ---------------------------------------------------------------------------
' Constants used for Windows NT Only in this module
' ---------------------------------------------------------------------------
  Private Const SE_SECURITY_NAME         As String = "SeSecurityPrivilege"
  Private Const SE_REMOTE_SHUTDOWN_NAME  As String = "SeRemoteShutdownPrivilege"
  Private Const SE_SHUTDOWN_NAME         As String = "SeShutdownPrivilege"
  Private Const SE_PRIVILEGE_ENABLED     As Long = &H2
  Private Const READ_CONTROL             As Long = &H20000
  Private Const STANDARD_RIGHTS_ALL      As Long = &H1F0000
  Private Const STANDARD_RIGHTS_EXECUTE  As Long = READ_CONTROL
  Private Const STANDARD_RIGHTS_READ     As Long = READ_CONTROL
  Private Const STANDARD_RIGHTS_REQUIRED As Long = &HF0000
  Private Const STANDARD_RIGHTS_WRITE    As Long = READ_CONTROL
  Private Const SYNCHRONIZE              As Long = &H100000
  Private Const TOKEN_ASSIGN_PRIMARY     As Long = &H1
  Private Const TOKEN_DUPLICATE          As Long = &H2
  Private Const TOKEN_IMPERSONATE        As Long = &H4
  Private Const TOKEN_QUERY              As Long = &H8
  Private Const TOKEN_QUERY_SOURCE       As Long = &H10
  Private Const TOKEN_ADJUST_PRIVILEGES  As Long = &H20
  Private Const TOKEN_ADJUST_GROUPS      As Long = &H40
  Private Const TOKEN_ADJUST_DEFAULT     As Long = &H80
  Private Const TOKEN_ALL_ACCESS         As Long = STANDARD_RIGHTS_REQUIRED Or _
                                                   TOKEN_ASSIGN_PRIMARY Or _
                                                   TOKEN_DUPLICATE Or _
                                                   TOKEN_IMPERSONATE Or _
                                                   TOKEN_QUERY Or _
                                                   TOKEN_QUERY_SOURCE Or _
                                                   TOKEN_ADJUST_PRIVILEGES Or _
                                                   TOKEN_ADJUST_GROUPS Or _
                                                   TOKEN_ADJUST_DEFAULT
  Private Const TOKEN_READ               As Long = STANDARD_RIGHTS_READ Or _
                                                   TOKEN_QUERY
  Private Const TOKEN_WRITE              As Long = STANDARD_RIGHTS_WRITE Or _
                                                   TOKEN_ADJUST_PRIVILEGES Or _
                                                   TOKEN_ADJUST_GROUPS Or _
                                                   TOKEN_ADJUST_DEFAULT
  Private Const TOKEN_EXECUTE            As Long = STANDARD_RIGHTS_EXECUTE
  Private Const PROCESS_ALL_ACCESS       As Long = STANDARD_RIGHTS_REQUIRED Or _
                                                   SYNCHRONIZE Or &HFFF

' ---------------------------------------------------------------------------
' Type structures used for Windows NT Only in this module
' ---------------------------------------------------------------------------
  Private Type Large_Integer
      LowPart         As Long
      HighPart        As Long
  End Type
 
  Private Type LUID
      LowPart         As Long
      HighPart        As Long
  End Type

  Private Type TOKEN_PRIVILEGES
      PrivilegeCount  As Long
      Privileges      As LUID
      Attributes      As Long
  End Type

' ---------------------------------------------------------------------------
' All other declarations
' ---------------------------------------------------------------------------
  Private Declare Function AdjustTokenPrivileges Lib "advapi32.dll" _
          (ByVal TokenHandle As Long, ByVal DisableAllPrivileges As Long, _
          NewState As TOKEN_PRIVILEGES, ByVal BufferLength As Long, _
          PreviousState As TOKEN_PRIVILEGES, ReturnLength As Long) As Long
  
  Private Declare Function CloseHandle Lib "kernel32" _
          (ByVal hObject As Long) As Long
  
  Private Declare Function GetCurrentProcess Lib "kernel32" () As Long
  
  Private Declare Function GetDesktopWindow Lib "user32" () As Long
  
  Private Declare Function GetTokenInformation Lib "advapi32.dll" _
          (ByVal TokenHandle As Long, TokenInformationClass As Integer, _
          TokenInformation As Any, ByVal TokenInformationLength As Long, _
          ReturnLength As Long) As Long
  
  Private Declare Function GetWindowThreadProcessId Lib "user32" _
          (ByVal hwnd As Long, lpdwProcessId As Long) As Long
          
  Private Declare Function LookupPrivilegeValue Lib "advapi32.dll" _
          Alias "LookupPrivilegeValueA" (ByVal lpSystemName As String, _
          ByVal lpName As String, lpLuid As LUID) As Long
  
  Private Declare Function OpenProcess Lib "kernel32" _
          (ByVal dwDesiredAccess As Long, ByVal bInheritHandle As Long, _
          ByVal dwProcessId As Long) As Long

  Private Declare Function OpenProcessToken Lib "advapi32.dll" _
          (ByVal ProcessHandle As Long, ByVal DesiredAccess As Long, _
          TokenHandle As Long) As Long
  
  Private Declare Function TerminateProcess Lib "kernel32" _
          (ByVal hProcess As Long, ByVal uExitCode As Long) As Long
          
  Private Declare Function GetVersionEx Lib "kernel32" _
          Alias "GetVersionExA" _
          (lpVersionInformation As OSVERSIONINFO) As Long

  Private Declare Function ExitWindowsEx Lib "user32" _
          (ByVal uFlags As Long, ByVal dwReserved As Long) As Long

' ---------------------------------------------------------------------------
' Used To translate API errors into something I can understand  :-)
' ---------------------------------------------------------------------------
  Private Const FORMAT_MESSAGE_ALLOCATE_BUFFER  As Long = &H100
  Private Const FORMAT_MESSAGE_IGNORE_INSERTS   As Long = &H200
  Private Const FORMAT_MESSAGE_FROM_STRING      As Long = &H400
  Private Const FORMAT_MESSAGE_FROM_HMODULE     As Long = &H800
  Private Const FORMAT_MESSAGE_FROM_SYSTEM      As Long = &H1000
  Private Const FORMAT_MESSAGE_ARGUMENT_ARRAY   As Long = &H2000
  Private Const FORMAT_MESSAGE_MAX_WIDTH_MASK   As Long = &HFF
 
  Private Declare Function FormatMessage Lib "kernel32" _
          Alias "FormatMessageA" (ByVal dwFlags As Long, _
          lpSource As Any, ByVal dwMessageId As Long, _
          ByVal dwLanguageId As Long, ByVal lpBuffer As String, _
          ByVal nSize As Long, Arguments As Long) As Long

Private Function IsWindows2000() As Boolean

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim OSInfo As OSVERSIONINFO

' ---------------------------------------------------------------------------
' Is this operating system Windows 2000 (NT5)
' ---------------------------------------------------------------------------
  With OSInfo
       .dwOSVersionInfoSize = Len(OSInfo) ' Prepare the receiving area
       .szCSDVersion = Space$(MAX_SIZE)   ' use spaces not nulls (less errors)
       GetVersionEx OSInfo                ' Get the version information
       IsWindows2000 = CBool(.dwPlatformId = VER_PLATFORM_WIN32_NT And _
                             .dwMajorVersion > 4)
       m_blnWindows2000 = IsWindows2000
  End With
  
End Function

Private Function IsWindowsNT() As Boolean

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim OSInfo As OSVERSIONINFO

' ---------------------------------------------------------------------------
' Is this operating system Windows NT (3.5 to 4.0)
' ---------------------------------------------------------------------------
  With OSInfo
       .dwOSVersionInfoSize = Len(OSInfo) ' Prepare the receiving area
       .szCSDVersion = Space$(MAX_SIZE)   ' use spaces not nulls (less errors)
       GetVersionEx OSInfo                ' Get the version information
       IsWindowsNT = CBool(.dwPlatformId = VER_PLATFORM_WIN32_NT And _
                           .dwMajorVersion = 4)
       m_blnWindowsNT = IsWindowsNT
  End With
  
End Function

Private Sub SetShutdownPrivilege()

' ***************************************************************************
' Routine:       SetShutdownPrivilege
'
' Description:   When using an NT based operating system, you must make sure
'                that the user has proper authority to shut the system down.
'                Here, we grant temporary authority for this action.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 16-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngProcessHwd        As Long
  Dim lngTokenHwd          As Long
  Dim lngBufLen            As Long
  Dim lngLastError         As Long
  Dim lngRetCode           As Long
  Dim typLUID              As LUID
  Dim typTokenPriv         As TOKEN_PRIVILEGES
  Dim typPreviousTokenPriv As TOKEN_PRIVILEGES
  
' ---------------------------------------------------------------------------
' Get the handle for the current process
' ---------------------------------------------------------------------------
  lngProcessHwd = GetCurrentProcess()

' ---------------------------------------------------------------------------
' Check the authority to perform a shutdown of this PC
' ---------------------------------------------------------------------------
  lngRetCode = OpenProcessToken(lngProcessHwd, TOKEN_ALL_ACCESS, lngTokenHwd)
  lngLastError = Err.LastDllError        ' capture any error code
  
  ' See if we were successfull.  If not, call the error routine to
  ' translate the API error code into something meaningful.
  If lngRetCode = 0 Then
      MsgBox API_Error(lngLastError) & vbCrLf & _
             "Module:  clsReboot.SetShutdownPriviledge" & vbCrLf & _
             "API Call:  OpenProcessToken()", _
             vbExclamation + vbOKOnly, "Error detected"
      m_blnErrorDetected = True
      GoTo Clean_Up
  End If
  
' ---------------------------------------------------------------------------
' Read the current priviledges to shutdown this PC
' ---------------------------------------------------------------------------
  lngRetCode = LookupPrivilegeValue(vbNullString, SE_SHUTDOWN_NAME, typLUID)
  lngLastError = Err.LastDllError        ' capture any error code
  
  ' See if we were successfull.  If not, call the error routine to
  ' translate the API error code into something meaningful.
  If lngRetCode = 0 Then
      MsgBox API_Error(lngLastError) & vbCrLf & _
             "Module:  clsReboot.SetShutdownPriviledge" & vbCrLf & _
             "API Call:  LookupPrivilegeValue()", _
             vbExclamation + vbOKOnly, "Error detected"
      m_blnErrorDetected = True
      GoTo Clean_Up
  End If

' ---------------------------------------------------------------------------
' transfer old authority to hold area,  usually ignored but do it anyway
' ---------------------------------------------------------------------------
  typPreviousTokenPriv = typTokenPriv
  lngBufLen = Len(typPreviousTokenPriv)
  
' ---------------------------------------------------------------------------
' Prepare a new temporary authority
' ---------------------------------------------------------------------------
  With typTokenPriv
       .PrivilegeCount = 1                 ' just one allowed at this time
       .Privileges = typLUID
       .Attributes = SE_PRIVILEGE_ENABLED
  End With
  
' ---------------------------------------------------------------------------
' Reset the shutdown authority (one time only)
' ---------------------------------------------------------------------------
  lngRetCode = AdjustTokenPrivileges(lngTokenHwd, False, typTokenPriv, _
                                     lngBufLen, typPreviousTokenPriv, _
                                     lngBufLen)

  lngLastError = Err.LastDllError        ' capture any error code
  
  ' See if we were successfull.  If not, call the error routine to
  ' translate the API error code into something meaningful.
  If lngRetCode = 0 Then
      MsgBox API_Error(lngLastError) & vbCrLf & _
             "Module:  clsReboot.SetShutdownPriviledge" & vbCrLf & _
             "API Call:  AdjustTokenPrivileges()", _
             vbExclamation + vbOKOnly, "Error detected"
      m_blnErrorDetected = True
  End If
  
Clean_Up:
' ---------------------------------------------------------------------------
' Always close any open handles
' ---------------------------------------------------------------------------
  Call CloseHandle(lngTokenHwd)
  Call CloseHandle(lngProcessHwd)

End Sub

Public Sub ShutdownSystem(Optional intAction As Integer = 0)
                
' ***************************************************************************
' Routine:       ShutdownSystem
'
' Description:   This is the main routine called by an outside reference.
'
' Parameters:    intAction - Numeric representation of type of shutdown
'                            process to perform
'
' Special Note:  On some Windows 9x operating systems, if the action
'                requested is LOGOFF then the PC will power off.  This does
'                not happen to all machines, just some of them.  Any answers
'                out there?  Please write.
'
'                Example:  CompUSA 500 mhz and Dell 200 mhz will power off
'                          Dell 500 mhz will Logoff.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 16-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strMsg            As String
  Dim lngShutdownFlags  As Long
  Dim lngRetCode        As Long
  Dim lngLastError      As Long
  Dim blnWindows9x      As Boolean
  
' ---------------------------------------------------------------------------
' Initialize local variables
' ---------------------------------------------------------------------------
  lngShutdownFlags = 0
  blnWindows9x = True        ' initialize to TRUE
  
' ---------------------------------------------------------------------------
' Make sure we have enabled the privilege to shutdown for this process if
' we're running NT:
' ---------------------------------------------------------------------------
  If IsWindows2000 Then
      blnWindows9x = False   ' Reset flag, not a Win9x operating system
      SetShutdownPrivilege
  ElseIf IsWindowsNT Then
      blnWindows9x = False   ' Reset flag, not a Win9x operating system
      SetShutdownPrivilege
  End If
      
' ---------------------------------------------------------------------------
' See if there were any errors up to this point then shutdown the application
' ---------------------------------------------------------------------------
  If m_blnErrorDetected Then
      StopTheProgram
      Exit Sub
  End If

' ---------------------------------------------------------------------------
' Determine the type of shutdown requested
' ---------------------------------------------------------------------------
  Select Case intAction
         Case 0
              lngShutdownFlags = EWX_LOGOFF
         Case 1
              lngShutdownFlags = EWX_SHUTDOWN
         Case 2
              lngShutdownFlags = EWX_REBOOT
         Case 3
              lngShutdownFlags = EWX_POWERDOWN
         Case Else
              lngShutdownFlags = EWX_LOGOFF
  End Select
  
' ---------------------------------------------------------------------------
' If performing a LOGOFF and this is not a NT based operating system, then
' close Windows Desktop Explorer so as to get a clean refresh.
' ---------------------------------------------------------------------------
  If (lngShutdownFlags = EWX_LOGOFF) And blnWindows9x Then
      TerminateExplorer              ' close the desktop explorer
      lngShutdownFlags = EWX_LOGOFF  ' make sure the switch has not changed
  End If

' ---------------------------------------------------------------------------
' This is where we try to shutdown gracefully.
' ---------------------------------------------------------------------------
  lngRetCode = ExitWindowsEx(lngShutdownFlags, &HFFFF)
  
  ' if we have a problem, then force a shutdown
  If lngRetCode = 0 Then
      ' if a Win2k operating system, add the "Force if hung" parameter
      If m_blnWindows2000 Then
          lngShutdownFlags = lngShutdownFlags Or EWX_FORCEIFHUNG
      Else
          ' all others, just add the "Force" parameter
          lngShutdownFlags = lngShutdownFlags Or EWX_FORCE
      End If
      
      ' Force a shutdown and lose any data that was not previously saved.
      ' It is too late at this point to try to save anything.  Some of
      ' the processes have already started their shutdown events.
      lngRetCode = ExitWindowsEx(lngShutdownFlags, 0&)
  
      ' if there is still an error, then just force the system to shut
      ' down by any means possible.  Try to get to the point to prompt
      ' the user to turn off the PC.
      If lngRetCode = 0 Then
          If m_blnWindows2000 Then
              lngShutdownFlags = EWX_SHUTDOWN Or EWX_FORCEIFHUNG
          Else
              lngShutdownFlags = EWX_SHUTDOWN Or EWX_FORCE
          End If
        
          ' This time, try to get the prompt to turn off the PC
          lngRetCode = ExitWindowsEx(lngShutdownFlags, 0&)
      End If
  End If

End Sub

Private Sub TerminateExplorer()

' ***************************************************************************
' Routine:       TerminateExplorer
'
' Description:   Close Windows Explorer if the user has opted to logoff a
'                Windows 9x operating system.  Sometimes the system will
'                hang or take a long time to logoff the user.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 16-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' 29-APR-2001  Kenneth Ives  kenaso@home.com
'              modified and documented
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngProcessIDExplorer  As Long
  Dim lngExplorerHwd        As Long
  Dim lngHandle             As Long
  Dim lngRetCode            As Long
  Dim lngLastError          As Long
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  lngHandle = 0
  lngExplorerHwd = 0
  lngProcessIDExplorer = 0
  
' ---------------------------------------------------------------------------
' Get the desktop handle
' ---------------------------------------------------------------------------
  lngHandle = GetDesktopWindow()

' ---------------------------------------------------------------------------
' Get the explorer process ID handle
' ---------------------------------------------------------------------------
  Call GetWindowThreadProcessId(lngHandle, lngProcessIDExplorer)

' ---------------------------------------------------------------------------
' See if we successfully got the Windows Explorer Process ID
' ---------------------------------------------------------------------------
  If lngProcessIDExplorer > 0 Then
  
      ' Get the handle for Explorer
      lngExplorerHwd = OpenProcess(PROCESS_ALL_ACCESS, 0&, lngProcessIDExplorer)

      ' See if we captured the Windows Explorer handle
      If lngExplorerHwd > 0 Then
          
          ' Terminate Windows Explorer
          Call TerminateProcess(lngExplorerHwd, 1&)
      End If
  End If
  
' ---------------------------------------------------------------------------
' Always close any open handles
' ---------------------------------------------------------------------------
  Call CloseHandle(lngExplorerHwd)
  Call CloseHandle(lngHandle)

End Sub

Private Function API_Error(ByVal lngErrorCode As Long) As String

' ***************************************************************************
' Routine:       API_Error
'
' Description:   Format an API error code to something legible
'
' Parameters:    lngErrorCode - API error code
'
' Returns:       Formatted message string from the Windows translator (for
'                lack of a better term)
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 16-APR-2001  Kenneth Ives  kenaso@home.com
'              Wrote routine
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strBuffer As String
  Dim lngLength As Long
  
' ---------------------------------------------------------------------------
' Preload the buffer with spaces.  This reduces errors.  Some API routines
' see NULL values as a null terminated string and stop trying to fill the
' buffer after the first encounter.  While some API routines read the VB
' function String() as a series of null terminated strings.
' ---------------------------------------------------------------------------
  strBuffer = Space$(255)

' ---------------------------------------------------------------------------
' Return the error message associated with LastDLLError:
' ---------------------------------------------------------------------------
  lngLength = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM Or _
                            FORMAT_MESSAGE_IGNORE_INSERTS, _
                            0, lngErrorCode, 0&, strBuffer, _
                            Len(strBuffer), ByVal 0)
  
' ---------------------------------------------------------------------------
' Return the formatted message, if possible
' ---------------------------------------------------------------------------
  API_Error = Left$(strBuffer, lngLength)
               
End Function
